/*Header-MicMac-eLiSe-25/06/2007

    MicMac : Multi Image Correspondances par Methodes Automatiques de Correlation
    eLiSe  : ELements of an Image Software Environnement

    www.micmac.ign.fr


    Copyright : Institut Geographique National
    Author : Marc Pierrot Deseilligny
    Contributors : Gregoire Maillet, Didier Boldo.

[1] M. Pierrot-Deseilligny, N. Paparoditis.
    "A multiresolution and optimization-based image matching approach:
    An application to surface reconstruction from SPOT5-HRS stereo imagery."
    In IAPRS vol XXXVI-1/W41 in ISPRS Workshop On Topographic Mapping From Space
    (With Special Emphasis on Small Satellites), Ankara, Turquie, 02-2006.

[2] M. Pierrot-Deseilligny, "MicMac, un lociel de mise en correspondance
    d'images, adapte au contexte geograhique" to appears in
    Bulletin d'information de l'Institut Geographique National, 2007.

Francais :

   MicMac est un logiciel de mise en correspondance d'image adapte
   au contexte de recherche en information geographique. Il s'appuie sur
   la bibliotheque de manipulation d'image eLiSe. Il est distibue sous la
   licences Cecill-B.  Voir en bas de fichier et  http://www.cecill.info.


English :

    MicMac is an open source software specialized in image matching
    for research in geographic information. MicMac is built on the
    eLiSe image library. MicMac is governed by the  "Cecill-B licence".
    See below and http://www.cecill.info.

Header-MicMac-eLiSe-25/06/2007*/
#include "StdAfx.h"

extern void sortDescendPt2drY(vector<Pt2dr> & input);
extern double Variance(Im2D_REAL4 & aImgIn, double aMoy = 0, Pt2di aRab = Pt2di(0,0));


template <class T,class TB>
void ImReadFromTxt(std::string aName, Im2D<T,TB> & aImOut, int aSkipLine=0,bool aVario=0, string colSep="\t");

double VarLap(Im2D_REAL4 * aIm2D);

int ascii2tif(int argc,char ** argv)
{
    std::string aPat("Ree*.tif"), aColSep("\t");
    std::list<std::string> mLFile;
    bool aVario(true), aOutIsReal4(false);
    Pt2di aSz(1024,768);
    int aHLine(0);
    bool aC2K(false), aDebug(0), aOptris(0), aInt2Deg(0), aDeg2Int(0);

    ElInitArgMain
            (
                argc,argv,
                LArgMain()  << EAMC(aPat,"ascii image pattern, images generated either by irbis software (variocam hd thermal camera) or with SDKdirect optris library",eSAM_IsPatFile),
              LArgMain()
                << EAM(aVario,"Vario", true, "Are ascii generated by irbis soft for camera Variocam? Def true, exept if Optris=1.")
                << EAM(aC2K,"C2K", true, "Variocam: conversion from cercius value to kelvin value *100, in order to got integer value instead of float.")
                << EAM(aSz,"Sz", true, "Image size, default [1024,768] which is the size of variocam thermal imagery")
                << EAM(aOptris,"Optris", true, "Are ascii generated from optris camera? Def false. if true, set proper image size.")
                << EAM(aInt2Deg,"Int2Deg", true, "Optris: conversion of unsigned integer value (raw data) to degree?.")
                << EAM(aDeg2Int,"Deg2Int", true, "Optris: conversion of degree value to int2 raw data?.")
                << EAM(aHLine,"DropHL", true, "Number of Line to drop in the Header, default 10 (variocam ascii irbis soft)")
                << EAM(aColSep,"ColSep", true, "column separator, default= tabulation")
                << EAM(aDebug,"Debug", true, "Display much more info in order to help debugging process")
                 << EAM(aOutIsReal4,"ImReal4", true, "Output image is real4 tif images?")
                );
    if (aOptris) aVario=0;
    if (aVario & !EAMIsInit(&aC2K)) aC2K=true;
    if (aVario & !EAMIsInit(&aHLine)) aHLine=10;

    if (aOptris & !EAMIsInit(&aSz)) aSz=Pt2di(640,480);
    //if (aVario & !EAMIsInit(&aInt2Deg)) aInt2Deg=0;

    cInterfChantierNameManipulateur * aICNM = cInterfChantierNameManipulateur::BasicAlloc("./");
    // create the list of images starting from the regular expression (Pattern)
    mLFile = aICNM->StdGetListOfFile(aPat);

    std::cout << mLFile.size() << " ascii images detected\n";

    for (auto & aName : mLFile){

    std::cout << "Ascii to tif, image  " <<aName <<"\n";
    // crée une image Ram de taille adéquate

    int SzX(aSz.x), SzY(aSz.y);
    Im2D_REAL4 aImRAM(SzX,SzY);

    ImReadFromTxt(aName,aImRAM,aHLine,aVario,aColSep);

    if (aVario & aC2K)
    {

        std::cout << "conversion from celcius to Kelvin for Variocam images.\n";
        ELISE_COPY
                (
                    aImRAM.all_pts(),
                    100*(273.15+aImRAM.in()),
                    aImRAM.out()
                    );

    }

    if (aOptris & aInt2Deg)
    {

        std::cout << "conversion unsigned int Optris PI to degree value.\n";
        ELISE_COPY
                (
                    aImRAM.all_pts(),
                    (aImRAM.in()-1000)/10,
                    aImRAM.out()
                    );

    }

    if (aOptris & aDeg2Int)
    {

        std::cout << "conversion of degree value to unsigned int Optris PI raw value.\n";
        ELISE_COPY
                (
                    aImRAM.all_pts(),
                    (aImRAM.in()*10)+1000,
                    aImRAM.out()
                    );

    }


    std::string aNameTiff=aName.substr(0, aName.size()-3) + "tif";
    std::cout << "Writing of  " <<aNameTiff <<"\n";
    // sauve l'image

    if(aC2K | aInt2Deg | aOutIsReal4) {
    Tiff_Im aTiff
             (
                 aNameTiff.c_str(),
                 aImRAM.sz(),
                 GenIm::real4,
                 Tiff_Im::No_Compr,
                 Tiff_Im::BlackIsZero
             );

    ELISE_COPY
            (
                aImRAM.all_pts(),
                aImRAM.in(),
                aTiff.out()
                );
    } else {
        Tiff_Im aTiff
                 (
                     aNameTiff.c_str(),
                     aImRAM.sz(),
                     GenIm::int2,
                     Tiff_Im::No_Compr,
                     Tiff_Im::BlackIsZero
                 );

        ELISE_COPY
                (
                    aImRAM.all_pts(),
                    aImRAM.in(),
                    aTiff.out()
                    );

    }


    }

    return EXIT_SUCCESS;
}


template <class T,class TB>
void ImReadFromTxt(std::string aName, Im2D<T,TB> & aImOut, int aSkipLine,bool aVario,string colSep)
{
    // lecture du fichier ascii
    ifstream ascii(aName);
    std::string output;
    T aVal(0);
    // passe l'entête
    if (aSkipLine>0) for (int i(0); i<(aSkipLine-1);i++){ ascii >> output;}


    for (int l(0); l < aImOut.sz().y; l++)
    {
        // si le separator de colonne n'est pas une tabultation ou un espace, ca se gere pas de la meme manière
        if (colSep!="\t")
        {
            ascii >> output;
            //std::cout << "column separator is neither tab or space\n";
            char *aBuffer = strdup((char*)output.c_str());
            for (int col(0); col < aImOut.sz().x; col++)
            {
                Pt2di aPos(col,l);
                std::string aValStr("");
                // first call to strok not same syntax than other call
                if (col==0) {aValStr = strtok(aBuffer,colSep.c_str());} else {
                    aValStr = strtok( NULL, colSep.c_str() );
                }
                //std::cout << aValStr << "\n";
                FromString(aVal,aValStr);
                aImOut.SetR(aPos,aVal);
            }

        } else {

            for (int col(0); col < aImOut.sz().x; col++)
            {
                Pt2di aPos(col,l);
                // separator; tabulation
                ascii >> output;
                if (aVario) output[2]='.';
                FromString(aVal,output);
                aImOut.SetR(aPos,aVal);
            }
        }
    }
}


int main_ascii2tif(int argc,char ** argv)
{
   ascii2tif(argc,argv);
   return EXIT_SUCCESS;
}



// derawtisation de video thermique (optris et variocam): 1) surabondant (trop de frame) et 2) noisy (en particulier optris, effet de filé
// select only one frame among n consecutive  frame, choosen in compairing image bluriness
// export tiff only for these selected frames

int Test_ascii2tif_BlurinessSelect(int argc,char ** argv)
{

    string aDir = "./";
    string aPat, aPattern;
    int aSampling(32),aMinInter(5);
    bool aOptris(1), aDebug(0);
    Pt2di Sz(640,480);

    ElInitArgMain
    (
          argc,argv,
              LArgMain()  << EAMC(aPattern,"ascii image pattern, images generated either by irbis software (variocam hd thermal camera) or with SDKdirect optris library",eSAM_IsPatFile),
              LArgMain()
                          << EAM(aOptris,"Optris", true, "Are ascii generated by optris SDK direct script? Def true.")
                          << EAM(aSampling,"Sampling", true, "Pick up the best image every n images, n=Sampling")
                          << EAM(aMinInter,"Inter", true, "Minimum interval between two selected images, in number of images")
                          << EAM(Sz,"Sz", true, "Size of image")
                          << EAM(aDebug,"Debug", true, "Display more info in order to help debugging process")
    );

    SplitDirAndFile(aDir, aPat, aPattern);
    cInterfChantierNameManipulateur * aICNM = cInterfChantierNameManipulateur::BasicAlloc(aDir);
    vector<string>  aSetIm = *(aICNM->Get(aPat));

    int aOffset(0);
    std::cout << "Selection of the " << round_down(aSetIm.size()/aSampling) << " sharpest images \n";
    for (int aKSelImg=0; aKSelImg< round_down(aSetIm.size()/aSampling); aKSelImg++)
    {

       vector<Pt2dr> aVPair;
       // for each images in this image group, compute variance of laplacian as indicator of sharpness
       for (int aKImg=aKSelImg*aSampling+aOffset; aKImg<(aKSelImg+1)*aSampling; aKImg++)
       {
        string aImTxtName = aSetIm[aKImg];

        Im2D_REAL4 aIm(Sz.x,Sz.y);
        ImReadFromTxt(aImTxtName,aIm);
        if (aDebug) std::cout << "Compute Var for img " << aImTxtName << "...";
        double aVar = VarLap(&aIm);
        if (aDebug) std::cout << "done\n";
        Pt2dr aPair(double(aKImg), aVar);
        aVPair.push_back(aPair);
       }
       // choose the best image and export to tif file
       sortDescendPt2drY(aVPair);
       string aImTxtName = aSetIm[aVPair.at(0).x];
       cout << "+" << aImTxtName  << " selected             , var of laplacian " << aVPair.at(0).y << "\n";
       string aCom = "mm3d TestLib Ascii2Tif "
                           + aImTxtName
                           + " Optris=1 "
                           + " Debug=" + ToString(aDebug)
                           ;
       if (aDebug) std::cout << aCom << "\n";
       system_call(aCom.c_str());
       // ensure that successive images are not too close
       aOffset=(int) (aVPair.at(0).x+aMinInter - (aKSelImg+1)*aSampling);
       if (aOffset<0) aOffset=0;
       if (aDebug) std::cout <<" offset : " << aOffset << "\n";
    }

    return 0;
}



Fonc_Num GaussBlur(Fonc_Num f)
{
    Im2D_REAL8 aDenoise55(5,5,
                "1 4 7 4 1 "
                "4 16 26 16 4 "
                "7 26 41 26 7 "
                "4 16 26 16 4 "
                " 1 4 7 4 1"
                );
   return (1/273)*som_masq(f,aDenoise55);
}

double VarLap(Im2D_REAL4 * aIm2D)
{
    Im2D_REAL8 aLapl(3,3,
                        "0 1 0 "
                        "1 -4 1 "
                        " 0 1 0"
                   );

    Im2D_REAL8 aDenoise(3,3,
                        "0 1 0 "
                        "1 2 1 "
                        " 0 1 0"
                        );


    Pt2di aSzKer(round_up((aLapl.sz().x-1)/2), round_up((aLapl.sz().y-1)/2));


    Im2D_REAL4 aIm2D_DNs(aIm2D->sz().x, aIm2D->sz().y);
    // reduce noise

   // ELISE_COPY(aIm2D->all_pts(), GaussBlur(aIm2D->in_proj()), aIm2D_DNs.out());
     ELISE_COPY(aIm2D->all_pts(), som_masq(aIm2D->in(0),aDenoise), aIm2D_DNs.out());

    Im2D_REAL4 aIm2D_Lpl(aIm2D->sz().x, aIm2D->sz().y);

    ELISE_COPY(aIm2D_DNs.all_pts(),som_masq(aIm2D_DNs.in(0), aLapl), aIm2D_Lpl.out());

    double aVar = Variance(aIm2D_Lpl, 0, aSzKer);

    return aVar;
}




/*Footer-MicMac-eLiSe-25/06/2007

Ce logiciel est un programme informatique servant à la mise en
correspondances d'images pour la reconstruction du relief.

Ce logiciel est régi par la licence CeCILL-B soumise au droit français et
respectant les principes de diffusion des logiciels libres. Vous pouvezdouble VarLap(Im2D_REAL4 * aIm2D)
utiliser, modifier et/ou redistribuer ce programme sous les conditions
de la licence CeCILL-B telle que diffusée par le CEA, le CNRS et l'INRIA
sur le site "http://www.cecill.info".

En contrepartie de l'accessibilité au code source et des droits de copie,
de modification et de redistribution accordés par cette licence, il n'est
offert aux utilisateurs qu'une garantie limitée.  Pour les mêmes raisons,
seule une responsabilité restreinte pèse sur l'auteur du programme,  le
titulaire des droits patrimoniaux et les concédants successifs.

A cet égard  l'attention de l'ucApplitilisateur est attirée sur les risques
associés au chargement,  à l'utilisation,  à la modification et/ou au
développement et à la reproduction du logiciel par l'utilisateur étant
donné sa spécificité de logiciel libre, qui peut le rendre complexe à
manipuler et qui le réserve donc à des développeurs et des professionnels
avertis possédant  des  connaissances  informatiques approfondies.  Les
utilisateurs sont donc invités à charger  et  tester  l'adéquation  du
logiciel à leurs besoins dans des conditions permettant d'assurer la
sécurité de leurs systèmes et ou de leurs données et, plus généralement,
à l'utiliser et l'exploiter dans les mêmes conditions de sécurité.

Le fait que vous puissiez accéder à cet en-tête signifie que vous avez
pris connaissance de la licence CeCILL-B, et que vous en avez accepté les
termes.
Footer-MicMac-eLiSe-25/06/2007*/


